/*
 * Instructions for building BLCR with a cross-compiler.
 *
 * NOTE: Cross-compilation of BLCR is experimental at this point and may
 * not always go smoothly.  We are currently considering only systems for
 * which the cross-compilation tool-chain has been installed in what we
 * believe is a "standard" manner (tool-chain executables prefixed with
 * a host-tuple recognized by config.sub).
 *
 * We welcome your feedback on how these instructions and/or the script
 * output by this program can be made more general and more robust.
 *
 * In the instructions that follow, we use "Host" to refer to the platform
 * on which you plan to run BLCR and "Build" to refer to the platform on
 * which you plan to compile BLCR.
 *
 * 1. This file is a C source file.  On Build, compile it with the cross-
 *    compiler, using the options you need/want to pass at configure time (to
 *    get the same ABI, libc version, etc.).
 *    If your compilation fails with errors, you may need to disable building
 *    one or more of the tests.  To do so, try changing the CHECK_* settings
 *    below to disable the problematic test(s).  However, if you do so you'll
 *    need some other way to determine the test result.  So, you'll probably
 *    want to contact us at checkpoint@lbl.gov for assistance.
 *
 * 2. Run the built program on Host, saving the output to a file in the top-
 *    level BLCR source directory on Build.  Getting the executable from
 *    Build to Host and the output file back to Build are up to you - we have
 *    no way to portably automate this.
 *    In what follows, we'll assume you've named the file "cross-configure",
 *    but any name is fine.  However, it is crucial that the file be saved
 *    in the top-level source directory (the same directory that contains 
 *    "configure").
 *
 * 3. Edit the new cross-configure script for completeness, fixing anything
 *    you can identify as incorrect and adding manually-determined answers
 *    for any CHECK_* settings you disabled in step #1.
 *    Look for the string "%MISSING%" to identify anything that is required
 *    before you can move to the next step.
 *    The one thing you will always need to supply is the Host type.
 *    If using a standard installation of a gcc cross compiler, then the
 *    type string is the prefix of the cross-compiler, minus the final
 *    hyphen.  For instance, if in step #1 you build this file using
 *    i586-redhat-linux-gcc, then HOST_TYPE should be "i586-redhat-linux".
 *
 * 4. Make the new script executable: chmod +x cross-configure
 *
 * 5. Ensure that you have, on the Build system, configured kernel source
 *    for the Host system as described in doc/html/BLCR_Admin_Guide.html
 *    and in the FAQ entry "What if I my kernel sources are unconfigured?"
 *    in doc/html/FAQ/html.  If you need to use the instructions in the FAQ
 *    to configure your kernel sources, then you'll need to replace "make"
 *    with "make ARCH=<host_arch> CROSS_COMPILE=<tool_prefix>".
 *    You will also need, on the Build system, either the "System.map" file
 *    or the "vmlinux" file from the Host's kernel.
 *
 * 6. Read the BLCR Admin Guide (in doc/html) to understand what options you
 *    want or need to pass to "configure", but don't run configure yet!
 *    You must provide the option --with-linux=[DIR] and one of either
 *    --with-system-map=[FILE] or --with-vmlinux=[FILE].  Otherwise the
 *    configure logic will default to trying to use the currently running
 *    kernel on the Build machine!
 *    When considering a value for --prefix (or any other options that will
 *    determine placement of BLCR files), specify the values as they will
 *    appear in the Host's filesystem regardless of if/how they may appear
 *    on the Build machine.
 *
 * 7. On Build, run "cross-configure" with the same options you would have
 *    passed to "configure" (as determined in the previous step).
 *
 * 8. On Build, run "make" to build BLCR using the cross toolchain.
 *    Don't run "make install" yet; it is covered in the steps that follow.
 *    Don't try to run "make insmod" or "make check" on Build, since they
 *    use binaries that have been built for Host.
 *    [If the directory you built in is mounted in the same location on Host,
 *     then you might try running "make insmod check" on Host.  However, this
 *     is untested and may break in strange ways.  If you do try this, then
 *     please let us know how well or poorly it works out.]
 *
 * 9. When considering --prefix and related configure options above, we told
 *    you to consider only the paths as they appear in the filesystem of the
 *    Host system.  This is important because some of these paths get encoded
 *    in the programs and/or libraries.  However, the next step is to run
 *    the installation step on Build.  So, now we will assume that you have
 *    one of two situations.  If your situation falls somewhere in between,
 *    then we recommend using case B.
 *    A) The Host filesystem is mounted (writtable) on the Build machine,
 *       rooted at some single prefix such as /export/example.
 *    B) The Host filesystem is not mounted on the Build machine.
 *    On Build, run "make install DESTDIR=<PATH>" where the value of <PATH>
 *    is your mount point for case A, or any empty directory of your choosing
 *    for case B (something like /tmp/blcr-destdir is a good choice).
 *
 * 10. If your system fell into case A of the previous step, then your files
 *    are now present on the Host filesystem.  If your system fell into case
 *    B, then you now must use some mechanism of your choice to copy all the
 *    files in the chosen DESTDIR to the Host filesystem.
 *
 * 11. You should now connect to Host, and continue the steps in the BLCR
 *     Admin Guide for "Loading the Kernel Modules" and "Configuring Users'
 *     environments".
 *
 * Optional:
 *    Because of the way BLCR generates its man pages, cross-compiled builds
 *    don't have man pages.  If you need/want man pages on the Host system,
 *    then you can build BLCR natively on any system of your choosing and
 *    manually copy the resuting man pages to Host.  There are no platform-
 *    specific elements to the man pages.
 *
 * $Id: cross_helper.c,v 1.3 2007/07/13 01:17:59 phargrov Exp $
 */

/* BEGIN CHECK_* variables for optional configuration.  */
#define CHECK_STACK_DIRECTION 1
#define CHECK_SIGNUM 1
/* END optional configuration.
 * If you think you need to modify anything below this line,
 * then please contact us at checkpoint@lbl.gov.
 * 
 */

#include <stdio.h>
#include <stdlib.h>
#ifndef CHECK_STACK_DIRECTION
  #define CHECK_STACK_DIRECTION 1
#endif
#ifndef CHECK_SIGNUM
  #define CHECK_SIGNUM 1
#endif

static void die(const char *msg) {
	fprintf(stderr, msg);
	exit(1);
}

#if CHECK_STACK_DIRECTION
static void cross_stack_direction(void) {
	static char *addr = 0;
	auto char dummy;
	if (addr == 0) {
		addr = &dummy;
		cross_stack_direction();
		return;
	}
	printf("cross_stack_direction=%d\n", (&dummy > addr) ? 1 : -1);
}
#else
static void cross_stack_direction(void) {
	printf("cross_stack_direction=%%MISSING%%\n");
}
#endif

#if CHECK_SIGNUM
extern int __libc_allocate_rtsig(int);
static void cross_signum(void) {
	int i = __libc_allocate_rtsig(0);
	if (i <= 0) die("Unable to determine CR_SIGNUM\n");
	printf("cross_signum=%d\n", i);
}
#else
static void cross_signum(void) {
	printf("cross_signum=%%MISSING%%\n");
}
#endif

static void print_header(void) {
	puts(
		"# Automatically-generated file for cross-confguration of BLCR.\n"
		"# You will always need to perform at least some manual editing.\n"
		"# So, see the file \"cross_helper.c\" for instructions.\n"
		"\n"
		"# You always need to set this one manually.\n"
		"HOST_TYPE=%MISSING%\n"
		"\n"
	);
}

static void print_trailer(void) {
	puts(
		"extra_conf_args=\"cross_signum=$cross_signum cross_stack_direction=$cross_stack_direction\"\n"
		"\n"
		"srcdir=`dirname $0`\n"
		"if test ! -f \"$srcdir/configure\" ; then\n"
		"  echo \"The script $0 must be in the same directory as the configure script.\"\n"
		"  exit 1\n"
		"fi\n"
		"${srcdir}/configure --host=$HOST_TYPE --program-prefix='' $extra_conf_args \"$@\"\n"
	);
}


int main(void) {
	print_header();
	cross_stack_direction();
	cross_signum();
	print_trailer();
	return 0;
}

